################################################################################
#
# Revision History
#
# Date       Name            Description
# ====       ====            ===================================================
# 1/16/13    Ming-chung      Reverse Cairo package installation from CRAN
#11/17/2011	 Qian			 2 channel data use percentile threshold too.
# 3/4/11     Ming-chung      Modify installation of Cairo package to use r-forge
#                            since the cran version does not work on R 2.12.x.
# 8/19/10    Qian            Added gene data transform for single and dual channel data.
# 6/24/09    Qian            Created BRBHeatmap() function for Class Comparison,
#                            Gene Set Comparison, and Time Series Calculations.
################################################################################
#' @import bitops
#' @import Cairo
BRBHeatmap <- function(AnalysisName, pngfilename, xdata, arrayid, classid, geneid,
    ArrayToolsPath, isSingleChannel = FALSE, paired = FALSE, DoGroupComparison = FALSE, GenesetName = "",
    HeatmapthresholdPercentileLow	= 20,	#Low threshold percentile for intensities of heatmaps.
    HeatmapthresholdPercentileHigh =	80,	#High threshold percentile for intensities of heatmaps.
    MinColor2	= 16710856,	#Color for minimum values in 2 color heatmaps.
    MaxColor2	= 10485760,	#Color for maximum values in 2 color heatmaps.
    MinColor3	= 16711680,	#Color for minimum values in 3 color heatmaps.
    MidColor3	= 0,	#Color for middle values in 3 color heatmaps.
    MaxColor3	= 33023,	#Color for maximum values in 3 color heatmaps.
    MissColor	= 10921638	#Color for missing values in heatmaps.
) {
# Function:
#	Generate heatmaps for three analysis methods: ClassComparison, GenesetComparison, TimeCourse.
# Input:
#	AnalysisName: This heatmap function works for three analysis methods with key words: ClassComparison, GenesetComparison, TimeCourse.
#	xdata: Genes are in rows, Arrays are in columns. It works for 1 gene or 1 array.
#	arrayid: Array IDs.
#	classid: Class names.
#	geneid: Gene symbols.
#	ArrayToolsPath: Installed ArrayTools system path.
#	isSingleChannel = FALSE: Single channel data indicator. Default is no single channel data.
#	paired = FALSE: Paired samples indicator. Default is no paired samples.
#	GenesetName = "": Gene set name. Default is an empty string.
# Output:
#	pngfilename: Path and file name of the generated heatmap.

    # Preference file.
  #  Prefs <- read.table(paste(ArrayToolsPath, "Prefs/Preferences.txt", sep="/"), header=T, as.is=T, sep="\t", row.names=NULL)
#	#Qian 11/17/2011 2 channel data use percentile threshold too.
 #   #TruncThresh <- as.numeric(Prefs$Value[Prefs$Variable == "Heatmapthreshold"])



    # Ting 7/16/19
    # TruncThreshPercentileLow <- as.numeric(Prefs$Value[Prefs$Variable == "HeatmapthresholdPercentileLow"])
    # TruncThreshPercentileHigh <- as.numeric(Prefs$Value[Prefs$Variable == "HeatmapthresholdPercentileHigh"])
    # MinColor2 <- as.numeric(Prefs$Value[Prefs$Variable == "MinColor2"])
    # MaxColor2 <- as.numeric(Prefs$Value[Prefs$Variable == "MaxColor2"])
    # MinColor3 <- as.numeric(Prefs$Value[Prefs$Variable == "MinColor3"])
    # MidColor3 <- as.numeric(Prefs$Value[Prefs$Variable == "MidColor3"])
    # MaxColor3 <- as.numeric(Prefs$Value[Prefs$Variable == "MaxColor3"])
    # MissColor <- as.numeric(Prefs$Value[Prefs$Variable == "MissColor"])

    TruncThreshPercentileLow <- HeatmapthresholdPercentileLow
    TruncThreshPercentileHigh <- HeatmapthresholdPercentileHigh

    # Data
    arrayid2 <- as.character(arrayid)
    classid2 <- as.character(classid)
    geneid2 <- as.character(geneid)

    narrays <- ncol(xdata)                       #--> number of arrays
    ngenes <- length(geneid2)                         #--> number of genes
    xdata[xdata < -999] <- NA

	# Scale or/and center the gene data.
#   AnalysisHeatmapDataTransform = FALSE: Perform gene data transform. Default is "not perform".
#	AnalysisHeatmapDataScale = FALSE: Scale gene data for heatmaps. Default is "not scale".
#	AnalysisHeatmapDataCenter = FALSE: Center gene data for heatmaps. Default is "not center".
	if(AnalysisName == "TimeCourse") {
		AnalysisHeatmapDataTransform = FALSE
		AnalysisHeatmapDataScale = FALSE
		AnalysisHeatmapDataCenter = FALSE
	} else if(isSingleChannel) {
		AnalysisHeatmapDataTransform = TRUE
		AnalysisHeatmapDataScale = TRUE
		AnalysisHeatmapDataCenter = TRUE
	} else {
		AnalysisHeatmapDataTransform = TRUE
		AnalysisHeatmapDataScale = FALSE
		AnalysisHeatmapDataCenter = TRUE
	}

	if(AnalysisHeatmapDataTransform) {
		imdata <- t(xdata)
		xdata <- scale(imdata, center = AnalysisHeatmapDataCenter, scale = AnalysisHeatmapDataScale)
		xdata <- t(xdata)
	}

    if(ngenes == 1) {
		#if(!is.matrix(xdata)) dim(xdata) <- c(ngenes, narrays)
		xdata <- as.matrix(xdata)
	}

    # Colors.
    # Use R package bitops to do bitwise comparison: convert Long to Hex for RGB color.
#    source(paste(ArrayToolsPath, "/R/BiocUtil.R", sep=""), echo = FALSE)
 #   if(!LoadRPackage("bitops")) stop("Could not download the R package, bitops.")
    MinColor2Hex <- rgb(bitAnd(MinColor2,255), bitAnd(MinColor2 %/% 256, 255), bitAnd(MinColor2 %/% 256^2, 255), maxColorValue = 255)
    MaxColor2Hex <- rgb(bitAnd(MaxColor2,255), bitAnd(MaxColor2 %/% 256, 255), bitAnd(MaxColor2 %/% 256^2, 255), maxColorValue = 255)
    MinColor3Hex <- rgb(bitAnd(MinColor3,255), bitAnd(MinColor3 %/% 256, 255), bitAnd(MinColor3 %/% 256^2, 255), maxColorValue = 255)
    MidColor3Hex <- rgb(bitAnd(MidColor3,255), bitAnd(MidColor3 %/% 256, 255), bitAnd(MidColor3 %/% 256^2, 255), maxColorValue = 255)
    MaxColor3Hex <- rgb(bitAnd(MaxColor3,255), bitAnd(MaxColor3 %/% 256, 255), bitAnd(MaxColor3 %/% 256^2, 255), maxColorValue = 255)
    if(isSingleChannel) {
		colors <- c(MinColor2Hex, MaxColor2Hex)
	} else {
		colors <- c(MinColor3Hex, MidColor3Hex, MaxColor3Hex)
    }
    mycols <- RGBColors(colors)

    # Thresholding. If TruncThresh <= 0, use the data range.
    DataRange <- range(xdata, na.rm=T)
	#Qian 11/17/2011 2 channel data use percentile threshold too.
	#if(isSingleChannel) {
		if(TruncThreshPercentileLow <= 0 || TruncThreshPercentileHigh <=0) {
			TruncLow <- DataRange[1]
			TruncHigh <- DataRange[2]
		} else {
            TruncTemp <- quantile(xdata, probs=c(TruncThreshPercentileLow, TruncThreshPercentileHigh)/100, na.rm=TRUE)
			TruncLow <- min(TruncTemp, na.rm=TRUE)
            TruncHigh <- max(TruncTemp, na.rm=TRUE)
			xdata [xdata < TruncLow] <- TruncLow
			xdata [xdata > TruncHigh] <- TruncHigh    #--> Threshold data
		}
	#} else {
	#	if(TruncThresh <= 0) {
	#		TruncLow <- DataRange[1]
	#		TruncHigh <- DataRange[2]
	#	} else {
	#		TruncLow <- -TruncThresh
	#		TruncHigh <- TruncThresh
	#		xdata [xdata < TruncLow] <- TruncLow
	#		xdata [xdata > TruncHigh] <- TruncHigh    #--> Threshold data
	#	}
	#}

    # Rotate expression data.
    imdata <- t(xdata)                 # Jacob: clock-wise rotating expression data because in the R-function
    if(ngenes > 1 && (AnalysisName == "ClassComparison" || AnalysisName == "TimeCourse")) {
        imdata <- imdata[,ngenes:1]    #        image the resulting heatmap is counter-clockwise rotated.
        geneid2 <- geneid2[ngenes:1]   # Jacob: because of counter-clock wise rotation in image()
		if(narrays == 1) dim(imdata) <- c(narrays, ngenes)
    }

    # Width and height.
    MaxWidth <- 2000
    MaxHeight <- 15000
    SquareSize <- 10    #--> Hard coding width, height, multiplicative factor for gene square

    widthz <- narrays * SquareSize + 300
    if(AnalysisName == "GenesetComparison") {
        heightz <- ngenes * SquareSize + 358
    } else {
        heightz <- ngenes * SquareSize + 330
    }

    widthz <- min(widthz, MaxWidth)
    heightz <- min(heightz, MaxHeight)

    # Class label --- so that there is only one of each class label on the plot
    CC <- length(unique(classid))                 #--> number of classes
    ncsp <- as.numeric(as.vector(table(classid))) #--> number of replicates per class
    ii <- 0
    for(i in 1:CC){
      if(ncsp[i] > 1){
        for(j in 2:ncsp[i]){
          classid2[ii+j] <- ""
        }
      }
      ii <- ii + ncsp[i]
    }

    # Creates color ribbon, sets basic limits
    xlimz <- c(1-0.5,narrays+0.5)
    ylimz <- c(1-0.5,ngenes+0.5)
    zlimz <- c(TruncLow,TruncHigh)

    # Heatmap graph.
 #   source(paste(ArrayToolsPath,"/R/BiocUtil.R", sep=""), echo = FALSE)
#	if(!LoadRPackage("Cairo")) stop("Could not download the R package, Cairo.")
    CairoPNG(filename = pngfilename, width = widthz, height = heightz, pointsize=10)
    if(AnalysisName == "GenesetComparison") {
        layout(matrix(c(1,1,2,2,3,0),3,2,byrow=T), widths=c(lcm(8),1), heights=c(lcm(1),1,lcm(2.2)))
        par(las = 1, mar = c(0, 0, 0, 0) + 0.1, xpd = FALSE, cex=1)
        plot.new()
        text(0, 0.5, paste("GeneSet:", GenesetName, sep=""), cex=1.5, adj=0)
    } else {
        layout(matrix(c(1,1,2,0),2,2,byrow=T), widths=c(lcm(8),1), heights=c(1,lcm(2.2)))
    }
    LabelLength = 11
    par(las = 2, mar = c(LabelLength, 0, LabelLength, LabelLength) + 0.1, xpd = FALSE, cex=1)
    image(x=1:narrays,y=1:ngenes,z=imdata,xlim=xlimz,ylim=ylimz,zlim=zlimz,col=mycols,axes=FALSE,xlab="",ylab="")

    # Label axes.
    if(paired && !DoGroupComparison) {
        arrayid2 <- arrayid2[1:narrays]
        axis(side=1,at=1:narrays,labels=substring(arrayid2,1,20),tick=FALSE)
    } else {
        axis(side=1,at=1:narrays,labels=substring(arrayid2,1,20),tick=FALSE)
        axis(side=3,at=1:narrays,labels=substring(classid2,1,20),tick=FALSE)
    }
    axis(side=4,at=1:ngenes,labels=substring(geneid2,1,20),tick=FALSE)

    # Missing values. If R doesn't know the color name, the hexadecimal number of the color will assign to MissingColor.
    MissColorHex <- rgb(bitAnd(MissColor,255), bitAnd(MissColor %/% 256, 255), bitAnd(MissColor %/% 256^2, 255), maxColorValue = 255)
    MissingColor <- unlist( colors.name(MissColorHex))[1]
    if(is.na(MissingColor)) MissingColor <- MissColorHex
    if (any(is.na(imdata))) {
        na.temp <- matrix(NA, nr=narrays, nc=ngenes)
        na.temp[is.na(imdata)] <- 2
        image(1:narrays, 1:ngenes, na.temp, add=T, col=MissColorHex)
    }

    # Vertical class lines.
    Divider <- "yellow"
    if(!(paired && !DoGroupComparison)) {
        if(!is.null(Divider) && CC>1){
            for(i in 1:(CC-1)){
                abline(v=sum(ncsp[1:i],0.5),col=Divider)
            }
        }
    }

    # Legend.
    par(las = 1, mar = c(3, 0, 1, 0) + 0.1, xpd = FALSE, cex=1)
    lenscale <- 100
    image(as.matrix(seq(TruncLow,TruncHigh,length=lenscale)),col=mycols,bty="n",xaxt="n",yaxt="n")
    mtext(round(c(TruncLow, TruncHigh),1), side=3, line=0.1, adj=c(0,1))
	if(AnalysisName == "TimeCourse") {
		mtext("Normalized value", side=3, line=0.2, adj=0.5)
	} else if(isSingleChannel) {
		mtext("Centered & scaled value", side=3, line=0.2, adj=0.5)
	} else {
		mtext("Centered log ratio", side=3, line=0.2, adj=0.5)
	}
    mtext(paste("Data range before thresholding: ", round(DataRange[1],1), " to ", round(DataRange[2],1), '.',sep=""), side=1, line=0.8, adj=0)
    mtext(paste("Missing values are in color ", '\"', sub("gray[0-9]+","gray",MissingColor), '\".',sep=""), side=1, line=1.8, adj=0)
    dev.off()
}

RGBColors <- function(color) {
# n: number of color (interval) bewteen MaxColor and MidColr (equal to number of intervals between MidColr and MinColor).
# color: specify maximum, middle, minimum colors; one channel data only have maximum and minimum colors.
#        Gray scale: color <- c("white", "black")
#        3 colors: color <- c("green", "black", "red")
    x <- col2rgb(color)/255
    n <- 50
    if(length(color) == 2) {
        r <- c(seq(x[1,1], x[1,2], length=2*n-1))
        g <- c(seq(x[2,1], x[2,2], length=2*n-1))
        b <- c(seq(x[3,1], x[3,2], length=2*n-1))
    } else if(length(color) == 3) {
        r <- c(seq(x[1,1], x[1,2], length=n), seq(x[1,2], x[1,3], length=n)[-1])
        g <- c(seq(x[2,1], x[2,2], length=n), seq(x[2,2], x[2,3], length=n)[-1])
        b <- c(seq(x[3,1], x[3,2], length=n), seq(x[3,2], x[3,3], length=n)[-1])
    } else {
        stop("An error has occurred in the function, RGBColors.")
    }
    mycols <- rgb(r, g, b)
    return(mycols)
}

# colors.hex and colors.name convert color names to hex and visa versa
# note that each color has a unique hex code but each hex code may have
# more than one color
colors.hex <- function( x=colors() ) {
    color.hex <- function(x) do.call( "rgb", as.list(col2rgb(x)/255) )
    sapply( x, color.hex )
}

colors.name <- function( x ) {
    color.name <- function( x ) colors()[ colors.hex() == x ]
    lapply( x, color.name )
}

