########################################################################################
##                                                                                    ##
## FUNCTION:  CreateHtmlTable                                                         ##
##                                                                                    ##
## THIS FUNCTION TAKES ANY DATAFRAME, AND CREATES THE HTML CODE FOR THAT TABLE.       ##
##                                                                                    ##
########################################################################################

CreateHtmlTable <- function( InputTable, TableCaption="", ClassName=NULL, IncludeGeneName=FALSE) {

## INPUT:
##
##   InputTable     = Data frame or matrix to be converted to HTML table.
##   TableCaption   = Optional table caption.
##   ClassName      = A vector of class name (see lars) or a list (see lassoedPC).
##                    If a list, the name of each element will be shown on the 1st column.
##
## OUTPUT:
##
##   Returned value is a vector where each component can be written as a line into an HTML file.
##

  n <- nrow(InputTable)

  ColNames <- dimnames(InputTable)[[2]]
  ColNames <- gsub(pattern="\\.", replacement=" ", x=ColNames)

  IncludeClassName <- ifelse(is.null(ClassName), FALSE, TRUE)
  temp <- rep( "", n+1+1*IncludeClassName)  #Each row in vector will contain HTML code for one row in resulting table, plus header row.

  # Create HTML code for row numbers.
  if (IncludeGeneName) {
    InputTable <- as.matrix(cbind( paste("<TR><TD>", rownames(InputTable)), InputTable )) #Qian 6/2/09
  } else {
    InputTable <- as.matrix(cbind( paste("<TR><TD>", 1:n), InputTable )) #Qian 6/2/09
  }

  # Create HTML code for header row.
  temp[1] <- paste("<TR><TH>&nbsp;<TH>", paste(ColNames, collapse="<TH>")) #Qian 6/1/09
  nheader <- 0
  if (IncludeClassName) {
    if (!is.list(ClassName)) {
      temp[2] <- paste("<TR><TD><TD>", paste(ClassName, collapse="<TD>"))
	  nheader <- 1
	} else {
	  nheader <- length(ClassName)
      for(j in 1:nheader)  temp[1+j] <- paste("<TR><TD>", names(ClassName)[j], "<TD>", paste(ClassName[[j]], collapse="<TD>"))
    }	
  }	
  for (i in 1:n)  temp[i+1+1*nheader] <- paste(InputTable[i,], collapse="<TD>")

  HtmlCode <- c(
      "<table border='1' class='sortable'>" #Qian 5/29/09 sortable
    , "<CAPTION ALIGN=left>"
    , TableCaption
    , "</CAPTION>"
    , temp
    , "</TABLE>"
  )

  return(HtmlCode)

}
