CreatePathwayDataFile <- function(geneId,
                                  filter,
                                  PathwayMin = 5,
                                  PathwayMax = 1000,
                                  DoBiocarta = T,
                                  DoKegg = F,
                                  fromKEGGdb,
                                  Organism,
                                  WorkPath) {

  ArrayToolsPath <- path.package("GSEA")

  ## FUNCTION TO CREATE DATA FILES IN FORTRAN FOLDER:
  ##
  ## GeneSet_data.txt - First line contains: number of genelists, PathwayMax.
  ##               Each subsequent line represents a genelist, and contains:
  ##               Number of genes in the list, followed by indices of included genes.
  ## GeneSet_name.rda - R vector, where each component gives the short name of the genelist.
  ## GeneSetList.rda  - R vector, where each component contains indices of included genes,
  ##               one vector component per genelist.
  ##

  if (!DoBiocarta & !DoKegg) stop("At least one of Biocarta or KEGG must be selected!")

  # ## READ MATRIX OF GENE IDS.
  #
  # GBaccFile <- paste(ProjectPath, "/Annotations/Accession.txt", sep="")
  # UniGeneFile <- paste(ProjectPath, "/Annotations/UGCluster.txt", sep="")
  # SymbolFile <- paste(ProjectPath, "/Annotations/Symbol.txt", sep="")
  #
  # GBaccExists <- ifelse(file.access(GBaccFile) < 0, F, T)
  # UniGeneExists <- ifelse(file.access(UniGeneFile) < 0, F, T)
  # SymbolExists <- ifelse(file.access(SymbolFile) < 0, F, T)
  #
  # if (UniGeneExists & SymbolExists) {  #Note, GenBank identifier might be missing for some projects ???
  #   UniGene <- matrix(scan(file=UniGeneFile, sep="\t", skip=1, what="", quiet = T), ncol=2, byrow=T)[,2]
  #   UniGene <- gsub("@#$%&", "\\'", UniGene, fixed=T) # originally is gsub("@#$%&", "\\\\'", UniGene, extended=F)
  #   # modified for R 2.10.0   MLI 11/2/2009
  #   Symbol <- matrix(scan(file=SymbolFile, sep="\t", skip=1, what="", quiet = T), ncol=2, byrow=T)[,2]
  #   Symbol <- gsub("@#$%&", "\\'", Symbol, fixed=T)
  #   if (GBaccExists) {
  #     GBacc <- matrix(scan(file=GBaccFile, sep="\t", skip=1, what="", quiet = T), ncol=2, byrow=T)[,2]
  #     GBacc <- gsub("@#$%&", "\\'", GBacc, fixed=T)              #Replace by backslash-apostrophe combination.
  #     GeneIds <- data.frame(GBacc, UniGene, Symbol)
  #     names(GeneIds) <- c("Accession", "UGCluster", "Symbol")
  #   } else {
  #     GeneIds <- data.frame(UniGene, Symbol)
  #     names(GeneIds) <- c("UGCluster", "Symbol")            #Lori 8/31/10
  #   }
  #
  #   ##added probe reduction consideration. 1/21/2008.
  #   ##note GeneIds is NOT filtered.
  #
  #   ##previous code commented 1/21/2008
  #   #load(paste(WorkPath, "/FilterColumnCurrent.rda", sep=""))
  #   #indic <- FilterColumnCurrent
  #   #GeneIds <- GeneIds[indic,]
  #   #NumGenesInAnalysis <- nrow(GeneIds)
  #
  #   load(paste(ProjectPath, "BinaryData/ApplyFilter.rda", sep="/"))
  #   #load(paste(WorkPath, "ProbeReductionIndex.rda", sep="/"))
  #   load(paste(WorkPath, "GeneId1.rda", sep="/"))
  #   ProbeReductionIndex <- as.numeric(names(GeneId1[,1]))
  #   NewFilter <- c(1:length(Result$Filter))
  #   GeneIds <- GeneIds[NewFilter %in% ProbeReductionIndex,,drop=F]
  #   NumGenesInAnalysis <- nrow(GeneIds)
  #
  # } else {
  #   #Yong noted 1/21/2008: this part code seems to use when there is no gene annotation files.
  #   #however, since VB part now checks if existing annotation folder before proceeding for gene set analysis,
  #   #this part code seems never to be used again. obsolete?
  #
  #   load(paste(WorkPath, "/GeneIdNames.rda", sep=""))
  #   NumGeneIds <- length(GeneIdNames)
  #   load(paste(WorkPath, "/GeneId1.rda", sep=""))
  #   NumGenesInAnalysis <- length(GeneId1)
  #   GeneIds <- matrix("", nrow=NumGenesInAnalysis, ncol=NumGeneIds)
  #   GeneIds[,1] <- GeneId1
  #   if (NumGeneIds > 1) {
  #     for (i in 2:NumGeneIds) {
  #       load(paste(WorkPath, "/GeneId", i, ".rda", sep=""))
  #       GeneIds[,i] <- get(paste("GeneId",i,sep=""))
  #     }
  #   }
  #   GeneIds <- as.data.frame(GeneIds)
  #   names(GeneIds) <- GeneIdNames
  #
  #   ##added probe reduction consideration. 1/21/2008.
  #   ##note GeneIds is already filtered.
  #   load(paste(ProjectPath, "BinaryData/ApplyFilter.rda", sep="/"))
  #   #load(paste(WorkPath, "ProbeReductionIndex.rda", sep="/"))
  #   ProbeReductionIndex <- as.numeric(names(GeneId1[,1]))
  #   NewFilter <- c(1:length(Result$Filter))[Result$Filter==1]
  #   GeneIds <- GeneIds[NewFilter %in% ProbeReductionIndex,,drop=F]
  #   NumGenesInAnalysis <- nrow(GeneIds)
  # }
  #
  NumGenesInAnalysis <- sum(filter)
  GeneIds <- geneId[filter == 1,]

  for (loopvar in 1:2) {
    Folder <- ""
    if (loopvar==1 & DoBiocarta) Folder <- paste(ArrayToolsPath, "/Pathways/Biocarta", sep="")
    if (loopvar==2 & DoKegg) {
      if (fromKEGGdb)
        Folder <- paste(ArrayToolsPath, "/Pathways/KEGG", sep="") else
          Folder <- paste(ArrayToolsPath, "/Pathways/KEGG_new", sep="")
    }
    if (Folder=="") next
    if (tolower(Organism)!="human" & loopvar==2 & DoKegg) {DoKegg=F ; next} ## Ming-Chung added code to handle mouse pathways in KEGG

    ## READ VECTOR OF PATHWAY FILENAMES.
    Genelists <- list.files(path=Folder, recursive=T)  #Make sure all contents are files, not subfolders.
    Genelists <- Genelists[regexpr("/",Genelists) <= 0]
    #Amy added the following lines to select either the Human or Mouse pathways. 2/10/05.
    MouseIndic <- (substr(Genelists,1,2) == "m_")
    if (tolower(Organism) == "human") Genelists <- Genelists[!MouseIndic]
    if (tolower(Organism) == "mouse") Genelists <- Genelists[MouseIndic]

    FullFileName <- paste(Folder, Genelists, sep="/")
    NumGenelists <- length(Genelists)

    ## FIND GENE ID COLUMNS THAT MATCH BETWEEN THE DATASET AND THE PATHWAY GENELISTS.
    SubsetIdNames <- c("UGCluster", "Symbol", "Accession")  #Hard-code these names for simplicity, since they are fixed anyway, and the names() function annoyingly substitutes periods for spaces.                   #Lori 08/31/10
    ColumnMatch <- match(names(GeneIds), SubsetIdNames)
    if (sum(!is.na(ColumnMatch)) == 0) return("Your dataset does not contain any of the following identifier types: UGCluster, Symbol, or Accession.")

    ## MATCH GENE IDS AGAINST PATHWAYS.
    indic <- matrix(F, nrow=NumGenesInAnalysis, ncol=NumGenelists)
    GenelistHasMatch <- rep(F, NumGenelists)

    for (i in 1:NumGenelists) {
      if (file.exists(FullFileName[i])) {
        SubsetIds <- read.table(FullFileName[i], sep="\t", header=T)
        for (j in 1:ncol(GeneIds)) {
          if (!(is.na(ColumnMatch[j]))) {   #If GeneId column matches a column in the genelist.
            #6/7/07 Yong reverted back since Rich asked to match all gene identifiers.
            #if (names(GeneIds)[j] != "Gene symbol") next   #Amy added 9/1/06 to match ONLY using the gene symbol.
            SubsetIdsTemp <- as.character(SubsetIds[,ColumnMatch[j]])
            SubsetIdsTemp <- SubsetIdsTemp[SubsetIdsTemp != ""]  #Get rid of blank gene ids, because otherwise EVERY gene with an empty gene id field will match!
            indic[,i] <- (toupper(as.character(GeneIds[,j])) %in% toupper(SubsetIdsTemp))|(indic[,i])
          }
        }

        if ((sum(indic[,i]) >= PathwayMin) & (sum(indic[,i]) <= PathwayMax)) GenelistHasMatch[i] <- T
      }
    }

    ## KEEP ONLY THE GENELISTS WHICH HAD A MATCH.
    NumGenelists <- sum(GenelistHasMatch)
    if (NumGenelists == 0) stop("None of your genes were found to match any of the pathways!")
    Genelists <- Genelists[GenelistHasMatch]
    indic <- indic[ , GenelistHasMatch,drop=F] # Supriya modified function to pass as matrix when only 1 pathway matched 05/31/05

    ## GET GENE INDICES.
    index <- rep("", NumGenelists)
    GeneSets <- list(NULL)
    NumGenesInList <- rep(0, NumGenelists)
    for (i in 1:NumGenelists) {
      NumGenesInList[i] <- sum(indic[,i])
      index[i] <- paste((1:NumGenesInAnalysis)[indic[,i]], collapse=" ")
      GeneSets[[i]] <- (1:NumGenesInAnalysis)[indic[,i]]
    }

    if (loopvar==1) {
      indexBiocarta <- index
      GeneSetsBiocarta <- GeneSets
      NumGenelistsBiocarta <- NumGenelists
      NumGenesInListBiocarta <- NumGenesInList
      GenelistsBiocarta <- Genelists
    }

    if (loopvar==2) {

      GeneSetsKegg <- GeneSets
      indexKegg <- index
      NumGenelistsKegg <- NumGenelists
      NumGenesInListKegg <- NumGenesInList
      GenelistsKegg <- Genelists
    }
  } #end of loop

  if (DoBiocarta & DoKegg) {
    GeneSets <- c(GeneSetsBiocarta,GeneSetsKegg)
    index <- c(indexBiocarta, indexKegg)
    NumGenelists <- NumGenelistsBiocarta + NumGenelistsKegg
    NumGenesInList <- c(NumGenesInListBiocarta, NumGenesInListKegg)
    Genelists <- c(GenelistsBiocarta, GenelistsKegg)
  }

  ## OUTPUT FILES TO FORTRAN FOLDER.  (FOR NOW KEEP ORIGINAL GO FILENAMES, BUT LATER SHOULD CHANGE TO PATHWAY FILENAMES.)
  ListFileName <- paste(WorkPath, "/GeneSetList.rda", sep="")
  GeneSetList <- index
  save(GeneSetList, file=ListFileName)

  DataFileName <- paste(WorkPath, "/GeneSet_data.txt", sep="")
  FirstLine <- paste(NumGenelists, PathwayMax, collapse=" ")
  GeneIndices <- paste(NumGenesInList, index, sep=" ")
  cat(FirstLine, file=DataFileName, sep="\n", append=F)
  cat(GeneIndices, file=DataFileName, sep="\n", append=T)

  PathwayNamesFile <- paste(WorkPath, "/GeneSetName.rda", sep="")
  GeneSetName <- Genelists
  save(GeneSetName, file=PathwayNamesFile)
  return(GeneSets)

}

