# 4/18/17    Ting modified code to use EntrezID to generate KEGG pathway web links with map coloring (in the previous version, gene symbol was used)
# 8/11/2014  Ting added to support predicted TF target genelists
# 06/30/09   Ming-Chung revert back not to output gene list for gene set analysis.
# 5/7/09     Qian modified to show 3 significant digits for FDR.
# 5/7/09     Qian modified to show 2 significant digits for Geometric means and fold changes.
# 07/15/07	 Yong modified to use Bioconductor annotation package for GO
# 06/10/07   Yong re-organized gene set comparison analysis
# 05/15/07   Deyun added to remove the file folder for HTML output
# 04/24/07	 Yong modified to add Gene Set Global Test using Bioconductor package: globaltest.
# 01/09/07   Deyun modified to add links for Broad pathway
# 01/04/07   Deyun Remove the file path for GeneListComparison
# 12/7/06    Yong	modified to replace dget/dput with load/save for R2.4.
# 12/29/06   Ming-Chung modified to add Survival-GO analysis
# 10/17/2006 Deyun added for protein domain analysis
# 11/10/2006 Deyun added for transcription factor analysis
# 12/11/2006 Deyun added for MicroRNA analysis

###############################################################################################################
##                                                                                                           ##
## THIS FILE CONTAINS ONE FUNCTION (GetGenesHTML) WHICH IS CALLED BY THE CLASS COMPARISON,                   ##
## CLASS PREDICTION, SURVIVAL, AND QUANTITATIVE TRAIT ANALYSES.  THE OUTPUT OF THIS FUNCTION                 ##
## IS A LIST CONTAINING STRINGS WHICH CAN BE INSERTED INTO HTML OUTPUT FILES TO CREATE                       ##
## THE TABLE OF SIGNIFICANT GENESETS AND TABLE OF GENES WITHIN THOSE GENESETS (FOR GENE SET                  ##
## COMPARISON) OR THE TABLE OF 'SIGNIFICANT GENES' (FOR ALL OTHER ANALYSIS TYPES), AND THE GENE              ##
## ONTOLOGY OBSERVED v. EXPECTED TABLE (IF APPLICABLE).  THE CHROMOSOMAL DISTRIBUTION BARPLOT                ##
## IS CREATED AS A SIDE EFFECT (BUT NOT A RETURNED OBJECT) OF THIS FUNCTION.                                 ##
##                                                                                                           ##
##  Notes:                                                                                                   ##
##  - 'classifiers' and 'GenesFiltered' are defined in <GetXXXXResults.R> files.                             ##
##  - 'AnalysisType', 'IsSingleChannel', 'GeneListFileName' defined in <RunXXXX.R> scripts.                  ##
##  - RunPermP' seems to be used only for 'ClassComparison', and is defined elsewhere ??                     ##
##  - codeSurv' is defined in 'GetComparisonResults.R' file.                                                 ##
##  - CVsupport', 'methods' and 'SameN' are defined in 'GetPredictionResults.R' file.                        ##
##  - SampleCode', 'SampleGroup' and 'GeneExpData' are defined in 'GetBinaryTreePredictionResults.R' file.   ##
##  - CVsupport', 'coxcoef' are defined in 'GetRiskResults.R' file.                                             ##
##   'ncomp' and 'pHotelling' required for Hotelling test are defined in 'RunGOComparison.R',                ##
##                 'GetGOcomparison.R' and other gene set comparison file.                                   ##
##  - 'StatusVariable' is defined in <RunXXXX.R> script.                                                     ##
##  - 'genepairIndex' is defined in 'GetPredictionResults.R> file.                                           ##
##  - 'xikprime' is defined in 'GetPAMResults.R> file.                                                      ##
##  - genelistfilename parameter is used in analysis which outputs more than one gene list.                  ##
##    If it is not specified, it is defined as GeneListFileName.                                             ##
###############################################################################################################



outputClassname <- function(classname, classsize) { # a helper function, Yong 12/15/06
  str1 <- ""
  bclasssize <- !missing(classsize)
  for (i in 1:length(classname)) {
   str2 <- paste("<font size=2>Class ", i, ": ", "<I>", classname[i], "</I>",
                 if (bclasssize) paste0(" (", classsize[i], " samples)"), "</font>", sep="") # add classsize, MLI 5/16/17
   str1 <- paste(str1, str2, sep="")
   if (i < length(classname)) str1 <- paste(str1, "; ", sep="")
  }
  str1 <- paste(str1, ".", sep="")
}

#Qihao added codes to output a file for Ingenuity Pathway Analysis. 7/15/2013
CreateIPAOutput<-function(IngenuityOutput,GeneIdForIngenuity,ReferenceClass,UseCorrelationAsFoldChange){
		Output<-matrix(NA,ncol=1,nrow=nrow(GeneIdForIngenuity))

		#Use Gene Symbol as the Gene Identifer.
	#Qihao added codes to handle the case that project gene identifer worksheet doesn't have gene symbol. 2014/2/26
	if (any(grepl('symbol',colnames(GeneIdForIngenuity),ignore.case = TRUE))){
		Output[,1]<-as.character(GeneIdForIngenuity[,(grep('symbol',colnames(GeneIdForIngenuity),ignore.case = TRUE)[1])])

		#Check whether the inputted GeneID object contains "NA" instead of blank elements. 2014/3/5
		Output[is.na(Output),1]<-''
		#Use EntrezID as second order gene identifer
		Missindex<-(Output[,1]=='')
		if (any(Missindex)){
			#Qihao added codes to check whether the project annotation worksheet has EntrezID or not. 1/15/2014
			if (any(grepl('EntrezID',colnames(GeneIdForIngenuity),ignore.case = TRUE))){
			Output[Missindex,1]<-as.character(GeneIdForIngenuity[Missindex,(grep('EntrezID',colnames(GeneIdForIngenuity),ignore.case = TRUE)[1])])
			}
		}
		Output[is.na(Output),1]<-''
		Missindex<-(Output[,1]=='')
		if (any(Missindex)){
			if (any(grepl('UniqueID',colnames(GeneIdForIngenuity),ignore.case = TRUE))){
				Output[Missindex,1]<-as.character(GeneIdForIngenuity[Missindex,grep('UniqueID',colnames(GeneIdForIngenuity),ignore.case = TRUE)])
			}else{
				Output[Missindex,1]<-as.character(GeneIdForIngenuity[Missindex,grep('ProbeSet',colnames(GeneIdForIngenuity),ignore.case = TRUE)])
			}
		}
	}else{
			if (any(grepl('UniqueID',colnames(GeneIdForIngenuity),ignore.case = TRUE))){
				Output[,1]<-as.character(GeneIdForIngenuity[,grep('UniqueID',colnames(GeneIdForIngenuity),ignore.case = TRUE)])
			}else{
				Output[,1]<-as.character(GeneIdForIngenuity[,grep('ProbeSet',colnames(GeneIdForIngenuity),ignore.case = TRUE)])
			}
	}
		#Compute fold change
		ComputeFC<-function(index1,index2,Data=GeneIdForIngenuity){
			#look for the Geom mean column.
			GeoMindex<-grep("Geom mean*", names(Data),ignore.case = TRUE)
			ArrayOneIndex<-GeoMindex[index1]
			ArrayTwoIndex<-GeoMindex[index2]
			CFC<-(as.numeric(levels(Data[,ArrayOneIndex])[Data[,ArrayOneIndex]])/as.numeric(levels(Data[,ArrayTwoIndex])[Data[,ArrayTwoIndex]]))
			#Qihao modfied codes to handle NA Inf -Inf and NaN values. 6/12/2014
			CFC<-as.matrix(sapply(CFC,function(x){if (is.finite(x)){if((x)<1){-x^(-1)}else{x}}else{x}}),ncol=1)
			colnames(CFC)<-paste('Fold Change of ', classlabels[index1],' VS ',classlabels[index2],sep='')
			return(round(CFC,digits=3))
		}
		if (UseCorrelationAsFoldChange){
			#Use correlation as fold change since Ingenuity only take the direction of the inputted fold change value. The magnitude of inputted fold change values would not be used in IPA.
			IngenuityCorrelation<-as.numeric(levels(GeneIdForIngenuity[,grep('correlation',colnames(GeneIdForIngenuity),ignore.case = TRUE)])[GeneIdForIngenuity[,grep('correlation',colnames(GeneIdForIngenuity),ignore.case = TRUE)]])
			Output<-cbind(Output,IngenuityCorrelation)
			colnames(Output)[2]<-'Correlation coefficient'
		}else{
			if (ReferenceClass!='Automatically create the combination output'){
				ReferenceClassOrder<-which(classlabels==ReferenceClass)
				for (i in 1:(length(classlabels))){
					if (i != ReferenceClassOrder){
						Output<-cbind(Output,ComputeFC(i,ReferenceClassOrder))
					}
				}
			}else{
				#We assume that there are less than four classes in this data.
				ComOrder<-combn(length(classlabels),2)
				for (i in 1:ncol(ComOrder)){
					classlabels[ComOrder[1,i]]
					Output<-cbind(Output,ComputeFC(ComOrder[1,i],ComOrder[2,i]))
				}
			}
		}
		#Append P-value to the output
		#Qihao modified the code to output only Parametric p-value into the Ingenuity output text file when Permutation p-value column is given in the output HTML table.
		Output<-cbind(Output,sapply(as.character(GeneIdForIngenuity[,grep('p*value',colnames(GeneIdForIngenuity),ignore.case = TRUE)[1]]),function(x){if(x=='< 1e-07'){as.numeric(x=0.0000000)}else{as.numeric(x)}}))

		colnames(Output)[(ncol(Output))]<-gsub('<*.NOBR>','',(grep('p*value',colnames(GeneIdForIngenuity),ignore.case = TRUE,value=T)[1]))
		colnames(Output)[1]<-c('Gene Identifier')
		write.table(Output,file=IngenuityOutput,quote=FALSE,row.names = FALSE,col.names=TRUE,sep='\t')
}
