prepareGSEA <- function(expr,
                        filter,
                        surv = FALSE,
                        time = NULL,
                        status = NULL,
                        doqt = FALSE,
                        qt,
                        geneId,
                        #anno,
                        cls,
                        isPaired = FALSE,
                        pairID = NULL,
                        doGroupComparison = FALSE,
                        grpID = NULL,
                        hasDuplic = FALSE,
                        duplicID = NULL,
                        rvm = TRUE,
                        nperm.GSA = 200,
                        doBiocarta = TRUE,
                        doKEGG = FALSE,
                        fromKEGGdb = TRUE,
                        pathwayMin = 5,
                        pathwayMax = 1000,
                        organism,
                        alpha = 0.005,
                        geneSetGroup = c("pathway","TranscriptionFactor","microRNA"),
                        frommiRTarBasev4,
                        outputPath){
  # # Purpose:
  # #   Create <LR_data.txt>, <class_perm_params.txt>, <experiments.rda>, <geneIdNames.rda> <geneIdX.rda>
  # # See <WriteDataToFilesPrediction.R>.
  #
  # # experiments.rda (originally created from WriteDataToFilesPrediction.R)
  # if (any(is.na(expr))) {
  #   expr[is.na(expr)] <- -9999999
  # }
  geneSetGroup <- match.arg(geneSetGroup)
  ArrayToolsPath <- path.package("GSEA")
  HasDuplic <- as.logical(hasDuplic)
  DoGroupComparison <- as.logical(doGroupComparison)
  LikeGOComparison <- T
  IsPaired <- as.logical(isPaired)
  if (surv | doqt) IsPaired <- F #no paired option for survival gene set comparison or quantitative trait data
  if (doqt) rvm <- FALSE
  if (IsPaired) id <- pairID else id <- NULL
  ClassVariable <- cls
  if (DoGroupComparison) GroupVariable <- grpID
  arr1 <- expr
  ArrayIds <- colnames(expr)
  WorkPath <- outputPath
  StatusVariable <- status
  TimeVariable <- time
  gsanperm <- nperm.GSA
  DoRandomVariance <- rvm

  #****** from WriteDataToFiles.R ***********

  indic.keep <- rep(T,ncol(expr))
  if ((LikeGOComparison & !surv & !doqt)) {
    if (!HasDuplic & !IsPaired) {
      indic.keep <- ifelse(ClassVariable == -9999999, F, T)
    }
    # Mingchung modified to allow paired data 8/24/2012
    if (LikeGOComparison && !surv && !doqt && as.logical(DoGroupComparison)) {
      indic.keep <- ifelse(GroupVariable == '-9999999', F, T)
      if (IsPaired) id <- id[indic.keep]
    }
    if (!surv & !doqt) pheno <- ClassVariable[indic.keep]
    if (LikeGOComparison && as.logical(DoGroupComparison)) grp <- GroupVariable[indic.keep]
  }


  if ((LikeGOComparison & surv)) {
    if (!HasDuplic) {
      indic.keep <- ifelse(StatusVariable == -9999999|TimeVariable == -9999999, F, T)
      pheno <- paste(StatusVariable[indic.keep],TimeVariable[indic.keep],sep="ZZZ")
    }
  }
  ArrayIds <- ArrayIds[indic.keep]
  arr1 <- arr1[,indic.keep, drop=FALSE] # fix for one gene
  # gc() # added Ming-Chung 11/28/06

  ####################################################################################################################

  ## AVERAGE OVER DUPLICATES.

  block <- NULL

  if (HasDuplic & !IsPaired) {
    AvgDat <- AverageDuplic(arr1, duplicID, pheno, Block = block)
    arr1 <- AvgDat$mat
    pheno <- AvgDat$pheno
    ArrayIds <- AvgDat$id
    block <- AvgDat$block
    rm(AvgDat); #gc()
  }

  ## Check the condition for minimum # of arrays.
  ## Change the # of min arrays to 2 per class as per Rich's email. Also require
  ## the condiction to be checked after we average duplicates. 7/9/2007
  if ((LikeGOComparison & !surv & !doqt)) {
    if (any(table(pheno) < 2)) {
      stop("ERROR: Class comparison analysis will not be performed with fewer than 2 arrays in any class.")
    }
  }

  if (IsPaired) {
    #call AverageDuplic() to remove any unpaired data from arr1, noted by Yong 6/14/07
    AvgDat <- AverageDuplic(arr1, paste(id,pheno), pheno, id, RemoveUnpaired = T)
    #use <- for arr1
    arr1 <- AvgDat$mat
    pheno <- AvgDat$pheno
    id <- AvgDat$id
    rm(AvgDat); #gc()
    NumIDs <- length(unique(id))
    # Change the minimum number of paired obs from 3 to 2. 07/09/2007.
    if (NumIDs < 2) {
      stop("ERROR: Number of paired observations is fewer than 2.")
    }
  }

  ########


  if ((LikeGOComparison & !surv & !doqt)) {
    if (IsPaired) {
      ordercode <- order(pheno,id)
      id <- id[ordercode]
      checkpaired <- all(id[1:round(length(id)/2,0)] == id[round(length(id)/2+1,0):length(id)])
      checkpaired <- ifelse(checkpaired,1,0)
      if (checkpaired==0) {
        stop("ERROR: One of your experiments is not paired.")

      }
    } else {
      ordercode <- order(pheno)
    }
    #use <- for arr1
    arr1 <- arr1[ ,ordercode, drop=FALSE]
    # gc()
    pheno <- pheno[ordercode]
    ArrayIds <- ArrayIds[ordercode]
    # if (HasBlock) block <- block[ordercode]
    if (DoGroupComparison) grp <- grp[ordercode]
  }

  if ((LikeGOComparison & surv)) {
    phenoClass <- as.character((t(data.frame(strsplit(pheno,"ZZZ")))[,1]))
    phenoTime <- as.double(as.character((t(data.frame(strsplit(pheno,"ZZZ")))[,2])))
    ordercode <- order(phenoClass)
    #use <- for arr1
    arr1 <- arr1[ ,ordercode, drop = FALSE]
    gc()
    pheno <- pheno[ordercode]
    phenoClass <- phenoClass[ordercode]
    if (all(phenoClass == "0")) event <- 0 else if(all(phenoClass == "1")) event <- 1 else event <- 2
    phenoTime <- phenoTime[ordercode]
    ArrayIds <- ArrayIds[ordercode]
    ClassLevels <- unique(phenoClass)
    NumClassLevels <- length(ClassLevels)
  }

  ####################################################################################################################

  ## CHECK NUMBER OF ARRAYS IN EACH CLASS.

  if ((LikeGOComparison & !surv & !doqt)) {
    ClassLevels <- unique(pheno)
    NumClassLevels <- length(ClassLevels)

    for (i in 1:NumClassLevels) {
      NumGrp <- sum(pheno==ClassLevels[i])
      BadClass <- as.character(ClassLevels[i])
      if (NumGrp==0) {
        stop(paste("ERROR: The phenotype class '", BadClass, "' contains no arrays.", sep=""))
      }
      if (NumGrp==1 & LikeGOComparison) {
        stop(paste("ERROR: The phenotype class '", BadClass, "' contains only one array.", sep=""))

      }
    }

    if ((NumClassLevels != 2) & IsPaired) {
      stop("ERROR: Paired test cannot be performed because number of phenotype classes is not equal to 2.")
    }

    if (NumClassLevels < 2) {
      stop("ERROR: Number of phenotype classes is fewer than 2.")

    }



  }

  ####################################################################################################################

  ##Gene Set Comparison Analysis: Gene probe to Gene symbol reduction. added by Yong 1/17/08
  ##Obsolete in v3.9, MLI 1/11/2010
  #if (LikeGOComparison) {
  #  load(paste(WorkPath, "GeneId1.rda", sep="/"))
  #  FilteredIndex <- as.numeric(names(GeneId1[,1])) #the index is row number of the full set of gene probes
  #  load(paste(WorkPath, "ProbeReductionIndex.rda", sep="/"))
  #  filter[filter==1] <- as.numeric(FilteredIndex %in% ProbeReductionIndex)
  #}

  ####################################################################################################################

  ## COMPUTE CLASS AVERAGES AND OUTPUT TO .rda FILES.

  if ((LikeGOComparison & !surv & !doqt)) {
    if (!IsPaired) {
      for (i in 1:NumClassLevels) {
        arr.current <- arr1[filter == 1,pheno==ClassLevels[i],drop=F]
        meanLR <- rowMeans(arr.current, na.rm=T)
        save(meanLR, file=paste(WorkPath, "/meanLR", i , ".rda", sep=""))
      }
      rm(arr.current);
      # gc()
    } else {
      npairs <- length(pheno)/2
      diff <- arr1[filter == 1,1:npairs] - arr1[filter == 1,(npairs+1):(2*npairs)]
      meandiff <- rowMeans(diff, na.rm=T)
      save(meandiff, file=paste(WorkPath, "/meandiff.rda", sep=""))
    }
  }

  ####################################################################################################################

  ## OUTPUT EXPERIMENT DESCRIPTORS TO .rda FILE.

  if ((LikeGOComparison & !surv & !doqt)) {
    if (!IsPaired) {
      experiments <- data.frame(ExpId=ArrayIds, pheno=pheno)
    } else {
      experiments <- data.frame(ExpId=ArrayIds, pheno=pheno, id=id)
    }
  }


  if ((LikeGOComparison & surv))
    experiments <- data.frame(ExpId=ArrayIds, phenoClass=phenoClass, phenoTime=phenoTime)

  if ((LikeGOComparison & doqt))
    experiments <- data.frame(ExpId=ArrayIds, qt=qt)

  save(experiments, file=paste0(WorkPath, "/experiments.rda"))

  ####################################################################################################################

  ## OUTPUT "arr1" TO "LR_data.txt" FILE.

  for (j in 1:ncol(arr1)) {
    isna <- is.na(arr1[,j])
    #note to use <-, keep in mind that treating arr1 as global variable!
    if (any(isna)) {
      arr1[isna,j] <- -9999999
      # gc()
    }
  }

  if (rvm) {
    #note to use <-, keep in mind that treating arr1 as global variable!
    arr1 <- cbind(arr1,filter)
    #  gc()
  }

  if ( (LikeGOComparison & !surv & !doqt)) {
    temp <- sum(pheno==ClassLevels[1])
    for (i in 2:NumClassLevels) {
      temp <- paste(temp, sum(pheno==ClassLevels[i]))
    }
      cat(paste(length(unique(ClassLevels)), nrow(arr1), ifelse(IsPaired, 1, 0)), 0, "\n",
          # the final '0' will be ignored for class comparison. See survival case where it was really used.
          file=paste(WorkPath, "/LR_data.txt", sep=""), append=F)
      cat(temp, "\n",
          file=paste(WorkPath, "/LR_data.txt", sep=""), append=T)

  }

  if ((LikeGOComparison & surv)) {
    # ArrayTools allows all status=1, but not allows all status=0.
    # If sum(n)=n(1), Fortran will recognize that all status=1.
    temp <- sum(phenoClass==ClassLevels[1])
    if (NumClassLevels > 1) {
      for (i in 2:NumClassLevels) temp <- paste(temp, sum(phenoClass==ClassLevels[i]))
    }

      cat(length(unique(ClassLevels)),
          ifelse(LikeGOComparison, sum(filter==1), nrow(arr1)),
          ifelse(IsPaired, 1, 0), event, "\n",   # Add 'event' to distinguish the case when all data are events/censored. 11/6/2013
          file=paste(WorkPath, "/LR_data.txt", sep=""), append=F) # MLI 10/14/2009 fix a bug in Geneset + survival.
      cat(temp, "\n", file=paste(WorkPath, "/LR_data.txt", sep=""), append=T)
      write.table(paste(phenoTime,collapse=" "),file=paste(WorkPath, "/LR_data.txt", sep=""),
                  sep="\t", quote=F, row.names=F, col.names=F, append=T)


  }



    if (LikeGOComparison && surv) {
      # no RVM possibility, so just use filtered and probe reducted genes. It should match with the geneset_data.txt  MLI 10/14/2009
      write.table(arr1[filter==1,,drop=F], file=paste(WorkPath, "/LR_data.txt", sep=""), sep="\t", quote=F, row.names=F, col.names=F, append=T)
    } else {
      if (!doqt)
        write.table(arr1, file=paste(WorkPath, "/LR_data.txt", sep=""), sep="\t", quote=F, row.names=F, col.names=F, append=T)
    }





  ####################################################################################################################

  ## CREATE exprSet object for gene set comparison. Yong 4/18/07
  if (LikeGOComparison) {
    #   source.out<-try(source(paste(ArrayToolsPath,"/R/GenesetTests.R", sep=""), echo = FALSE))
    # if (class(source.out)=='try-error') {
    #   require(tcltk)
    #   tkmessageBox(message=paste('Error occurred in sourcing GenesetTests.R:', geterrmessage(), sep='\n'),icon="info",type="ok")
    # }

    if (rvm) {
      arr.current <- arr1[, -ncol(arr1), drop = FALSE] #remove the last column filter when DoRandomVariance is True
    } else {
      arr.current <- arr1
    }
    arr.current <- arr.current[filter==1,,drop=F] #now arr.current is the real filtered data matrix.
    if (!surv & !doqt)
      myPheno <- data.frame(pheno) else
      if (surv) myPheno <- data.frame(as.numeric(phenoClass), phenoTime) else #qt data
        myPheno <- data.frame(qt) # treat qt data as pheno??? TIng 7/26/19
    if (!CreateExprSetData(arr.current, myPheno, ArrayIds, WorkPath)) {
      stop("ERROR: exprSet object cannot be created for gene set comparison analysis.")

    }
    # Added MLI 9/13/a
    if (as.logical(DoGroupComparison)) {
      x <- arr.current
      y <- as.character(myPheno[,1])
      # check if the number of levels in both y and grp are 2.
      if (length(unique(grp)) != 2) {
        stop("ERROR: the analysis works only for two groups data.")

      }
      if (length(unique(y)) != 2 | any(table(y, grp)==1)) {
        #Qian 8/8/2012 Error message for current status, we may extend to multi-class.
        stop("ERROR: only 2-class or paired data is allowed.")

      }
      gsanperm <- as.integer(gsanperm)
      save(x, y, grp, ArrayIds, DoRandomVariance, gsanperm,
           arr1, filter,
           IsPaired, id,
           file=file.path(WorkPath, "GSEAgroupcomparison.rda"))
    }
  }

  #*********************************

  #create GeneSet_data.txt
#  CreatePathwayDataFile(ArrayToolsPath,geneId,filter,pathwayMin,pathwayMax,doBiocarta,doKegg,organism)
  if (geneSetGroup == "pathway")
    GeneSets <- CreatePathwayDataFile(geneId,filter,pathwayMin,pathwayMax,doBiocarta,
                                      doKEGG,fromKEGGdb,organism,WorkPath) else
    if (geneSetGroup == "TranscriptionFactor") {
      geneListDir <- paste0(path.package("GSEA"), "/Transcription_factor/TRED_", tolower(organism))
      UserGeneListFileNames <- paste0(geneListDir,"/",list.files(path = geneListDir,recursive = TRUE))
      GeneSets <- CreateGeneListDataFile(geneId,filter,UserGeneListFileNames,
                                         pathwayMin,pathwayMax,organism,WorkPath)
    } else if (geneSetGroup == "microRNA") {
      if (frommiRTarBasev4)
        geneListDir <- paste0(path.package("GSEA"), "/MicroRNA/v4/experimentally_verified_", tolower(organism)) else
          geneListDir <- paste0(path.package("GSEA"), "/MicroRNA/v7/experimentally_verified_", tolower(organism))
      UserGeneListFileNames <- paste0(geneListDir,"/",list.files(path = geneListDir,recursive = TRUE))
      GeneSets <- CreateGeneListDataFile(geneId,filter,UserGeneListFileNames,
                                         pathwayMin,pathwayMax,organism,WorkPath)
    }



 if (!doqt){
   # create class_perm_params.txt
   ParamLinear <- T # only for user genelist comparison: do exact permutation in Fortran program GeneSetComparison.exe (from GeneSetComparison.base)
   classperm <- c(alpha,
                  ifelse(rvm, 1, 0),
                  ifelse(surv, 1, 0),
                  1, # fixed random seed 123456789 is used if 1; 0 otherwise.
                  ifelse(ParamLinear, 1, 0)) # 1: linear approximation; 0: exact permutation
   cat(classperm, file=file.path(WorkPath, "class_perm_params.txt"), sep = "\n")
 }

  return(GeneSets)

}

