
#include "header.h"

double UNIFORMRV()  //uniformly generate samples from [0,1)
{
  return rand() / (RAND_MAX + 1.);
}

/* Generate n random samples from 0, 1, ..., N-1 without replacement;
 * Inputs:
 *  n: sample size
 *  N: populatiohn size
 * Return:
 *  s: n random samples
 * Reference: J.S. Vitter. An efficient algorithm from sequential random sampling.
 *            P.R-0634, INRIA. 1987. <inria-00075979>
 */
void sampleVitterD(int n,
                   int N,
                   int *s){

  int i,j,t,S,qu1,negalphainv,threshold,limit;
  i = 0;
  j = -1;
  qu1 = N - n + 1;
  negalphainv = -13;
  threshold = -negalphainv*n;

  double nreal, Nreal, ninv, nmin1inv, U, X, Vprime, y1, y2, top, bottom, negSreal, qu1real;
  nreal = n;
  Nreal = N;
  ninv = 1.0 / n;
  nmin1inv = 1.0 / (n-1);
  qu1real = Nreal - nreal + 1.0;
  Vprime = exp(log(UNIFORMRV())*ninv);

  while(n > 1 && threshold < N){
    nmin1inv = 1.0 / (-1.0 + nreal);
    while(1){
      while(1){
        X = Nreal * (-Vprime + 1.0);
        S = floor(X);
        if(S < qu1) break;
        Vprime = exp(log(UNIFORMRV()) * ninv);
      }
      U = UNIFORMRV();
      negSreal = -S;
      y1 = exp(log(U*Nreal/qu1real) * nmin1inv);
      Vprime = y1 * (-X / Nreal+1.0) * (qu1real / (negSreal + qu1real));
      if (Vprime <= 1.0) break;
      y2 = 1.0;
      top = -1.0 + Nreal;
      if (-1+n > S){
        bottom = -nreal + Nreal;
        limit = -S + N;
      }
      else{
        bottom = -1.0 + negSreal + Nreal;
        limit = qu1;
      }
      for (t = N-1; t >= limit; t--){
        y2=(y2 * top) / bottom;
        top--;
        bottom--;
      }

      if (Nreal / (-X + Nreal) >= y1 * exp(log(y2) * nmin1inv)){
        Vprime = exp(log(UNIFORMRV()) * nmin1inv);
        break;
      }
      Vprime = exp(log(UNIFORMRV()) * ninv);
    }
    j += S + 1;
    s[i++] = j;
    N = -S + (-1 + N);
    Nreal = negSreal + (-1.0 + Nreal);
    n--;
    nreal--;
    ninv = nmin1inv;
    qu1 = -S + qu1;
    qu1real = negSreal + qu1real;
    threshold += negalphainv;
  }

  if(n>1)
    sampleVitterA(n,N,j,s+i);
  else {
    S = floor(N * Vprime);
    j += S + 1;
    s[i++] = j;
  }

  return;

}

void sampleVitterA(int n,
                   int N,
                   int j,
                   int* s)
{
  int S, i;
  double V, quot, Nreal, top;
  Nreal = N;
  top = N - n;
  i = 0;

  while (n >= 2){
    V = UNIFORMRV();
    S = 0;
    quot = top / Nreal;
    while (quot > V){
      S++;
      top--;
      Nreal--;
      quot = (quot * top)/Nreal;
    }
    j+=S+1;
    s[i++]=j;
    Nreal--;
    n--;
  }
  S = floor(round(Nreal) * UNIFORMRV());
  j += S+1;
  s[i++] = j;

  return;
}

// for quantitative trait data
void geneSetEnrichmentQT(int *nvarx,// # number of genes
                         int *geneSet,
                         int *numGenesPerSet,
                         int *numSets,
                   //      int *randomSeed,
                         double *pvalueCorr,
                         double *pvalueSetLS,
                         double *pvalueSetKS,
                         double *LS,
                         double *KS){

  int i, j, k, nvar;
  nvar = *nvarx;
  // observed LS statistics
  LSStat(pvalueCorr, geneSet, numGenesPerSet, numSets[0], LS);
  KSStat(pvalueCorr, geneSet, numGenesPerSet, numSets[0], KS);

  /* The following re-sampling procedure randomly selects N genes
  * from the list of genes under analysis, where N is
  * the number of genes in one gene set.
  */
  int *sample;


  double *rLS;
  rLS = (double *) malloc(numSets[0]*NPM*sizeof(double));
  double *rKS;
  rKS = (double *) malloc(numSets[0]*NPM*sizeof(double));

  /*
   // when random seed is not fixed for each gene set in the resampling procedure
  //calculate all genes in the provided gene sets
  srand(randomSeed[0]);
  int numAllGenes = 0;
  for (i = 0; i < numSets[0]; i++)
    numAllGenes += numGenesPerSet[i];

  sample = (int *) malloc(numAllGenes*sizeof(int));
  int l = 0;
  for (i = 0; i < NPM; i++){ // number of permutations
    k = 0;
    for (j = 0; j < numSets[0]; j++){
      sampleVitterD(numGenesPerSet[j], nvar, &sample[k]);
      k += numGenesPerSet[j];
    }
    LSStat(pvalueCorr, sample, numGenesPerSet, numSets[0], &rLS[l]);
    KSStat(pvalueCorr, sample, numGenesPerSet, numSets[0], &rKS[l]);
    l += numSets[0];
  } // end of permutations

  // calculate p-values for each gene set
  for (i = 0; i < numSets[0]; i++){
    pvalueSetLS[i] = 0.0;
    pvalueSetKS[i] = 0.0;
    for (j = 0; j < NPM; j++){
      if (rLS[j*numSets[0]+i] > LS[i])
        pvalueSetLS[i] += 1.0;
      if (rKS[j*numSets[0]+i] > KS[i])
        pvalueSetKS[i] += 1.0;
    }
    pvalueSetLS[i] /= NPM;
    pvalueSetKS[i] /= NPM;
  }

   */

  // when random seed is fixed for each gene set in the resampling procedure
  int maxGS, l;
  maxGS = imax(numGenesPerSet, numSets[0]);
  sample = (int *) malloc(maxGS*NPM*sizeof(int));

  l = 0;
  for (j = 0; j < numSets[0]; j++){ // number of permutations
    //  srand(randomSeed[0]);
    GetRNGstate(); //initialize random seed
    k = 0;
    for (i = 0; i < NPM; i++){
      sampleVitterD(numGenesPerSet[j], nvar, &sample[k]);
      k += numGenesPerSet[j];
    }
    LSStat2(pvalueCorr, sample, numGenesPerSet[j], NPM, &rLS[l]);
    KSStat2(pvalueCorr, sample, numGenesPerSet[j], NPM, &rKS[l]);
    l += NPM;
  } // end of permutation

  PutRNGstate(); //save random seed

  for (i = 0; i < numSets[0]; i++){
    pvalueSetLS[i] = 0;
    pvalueSetKS[i] = 0;
  }

  // calculate p-values for each gene set
  for (j = 0; j < NPM; j++){
    for (i = 0; i < numSets[0]; i++){
      if (rLS[i*NPM+j] > LS[i])
        pvalueSetLS[i] += 1.0;
      if (rKS[i*NPM+j] > KS[i])
        pvalueSetKS[i] += 1.0;
    }
  }

  for (i = 0; i < numSets[0]; i++){
    pvalueSetLS[i] /= NPM;
    pvalueSetKS[i] /= NPM;
  }



  free(sample);
  free(rLS);
  free(rKS);

  return;
}

/* Calculate log score (LS) statistics;
 * Inputs:
 *  p: vector of p-values
 *  x: vector of indices for all sets
 *  n: vector of the size of each set
 *  len: number of LS statistics to be calculated
 * Return:
 *  s: vector of LS statistics for each set
 */

void LSStat(double *p,
              int *x,
              int *n,
              int len,
              double *LS){

  int i, j, p0, p1;
  for (i = 0; i < len; i++){
    p0 = 0;
    if (i == 0)
      p0 = 0;
    else{
      for (j = 0; j < i; j++)
        p0 += n[j];
    }
    p1 = p0 + n[i];

    LS[i] = 0;
    for (j = p0; j < p1; j++)
      LS[i] += -log(p[x[j]])/log(2.0);
    LS[i] /= n[i];

  }
  return;
}


double dmax(double *x,
          int n){

  double y = x[0];

  for (int i = 1; i < n; i++){
    if (x[i] > y)
      y = x[i];
  }
  return(y);

}

int imax(int *x,
            int n){

  int y = x[0];

  for (int i = 1; i < n; i++){
    if (x[i] > y)
      y = x[i];
  }
  return(y);
}


// sort a double vector in an increasing order
void dsort(double *x,
           int n,
           double *y){

  int i, j;
  double tmp;

  for (i = 0; i < n-1; i++){
    for (j = i + 1; j < n; j++){
      if (x[j] < x[i]){
        tmp = x[i];
        x[i] = x[j];
        x[j] = tmp;
      }
    }
  }

  for (i = 0 ; i < n; i++)
    y[i] = x[i];
}

/* Calculate Kolmogorov-Smirnov (KS) statistics;
 * Inputs:
 *  p: vector of p-values
 *  x: vector of indices for all sets
 *  n: vector of the size of each set
 *  len: number of KS statistics to be calculated
 * Return:
 *  s: vector of KS statistics for each set
 */

void KSStat(double *p,
            int *x,
            int *n,
            int len, // number of gene sets
            double *KS){

  double *po, *pt; // in an increasing order of *p
  int t = imax(n, len);
  po = (double *) malloc(t*sizeof(double));
  pt = (double *) malloc(t*sizeof(double));

  int i, j, p0, p1;
  for (i = 0; i < len; i++){

    p0 = 0;
    if (i > 0){
      for (j = 0; j < i; j++)
        p0 += n[j];
    }
    p1 = p0 + n[i];


    for (j = p0; j < p1; j++)
      pt[j-p0] = p[x[j]];

    dsort(pt,n[i],po);

    for (j = 0; j < n[i]; j++)
      pt[j] = (j+1.0)/n[i] - po[j];

    KS[i] = dmax(pt,n[i]);


  }

  free(po);
  free(pt);
  return;
}


/* Calculate Kolmogorov-Smirnov(KS) statistics; this function is used in geneSetEnrichmentCox when
 * the random seed is fixed for each gene set during the resampling procedure
 * Inputs:
 *  p: vector of p-values
 *  x: vector of indices for all sets
 *  n: the size of a gene set
 *  len: number of LS statistics to be calculated
 * Return:
 *  s: vector of LS statistics for each set
 */

void KSStat2(double *p,
             int *x,
             int n,
             int len,
             double *KS){

  int i, j;
  double *po, *pt;
  pt = (double *) malloc(n*sizeof(double));
  po = (double *) malloc(n*sizeof(double));
  for (i = 0; i < len; i++){
    KS[i] = 0;
    for (j = i*n; j < (i+1)*n; j++)
      pt[j-i*n] = p[x[j]];
    dsort(pt,n,po);
    for (j = 0; j < n; j++)
      pt[j] = (j+1.0)/n - po[j];
    KS[i] = dmax(pt,n);

  }
  free(pt);
  free(po);
  return;
}










/* Calculate log score (LS) statistics; this function is used in geneSetEnrichmentQT() when
 * the random seed is fixed for each gene set during the resampling procedure
 * Inputs:
 *  p: vector of p-values
 *  x: vector of indices for all sets
 *  n: the size of a gene set
 *  len: number of LS statistics to be calculated
 * Return:
 *  s: vector of LS statistics for each set
 */

void LSStat2(double *p,
             int *x,
             int n,
             int len,
             double *LS){

  int i, j;
  for (i = 0; i < len; i++){
    LS[i] = 0;
    for (j = i*n; j < (i+1)*n; j++)
      LS[i] += -log(p[x[j]])/log(2.0);
    LS[i] /= n;

  }
  return;
}
